////////////////////////////////////////////////////////////////////////////////
//Name:    Base TTF
//Purpose: Base functions for text rendering
////////////////////////////////////////////////////////////////////////////////
class base_ttf
{
////////////////////////////////////////////////////////////////////////////////
 public:
////////////////////////////////////////////////////////////////////////////////
//Name:    Load Font
//Purpose: Load a TTF font from file and set it up for rendering
//Notes:   Loading a font into a previously loaded font may cause memory leak
//         or segfault.
//Also:    Double pointer required for required pass by reference.
////////////////////////////////////////////////////////////////////////////////
 void load_font(TTF_Font **font,const char* font_file, int font_size)
 {if(!(*font=TTF_OpenFont(font_file,font_size)))
    {printf("CRITICAL EXISTANCE FAILURE: TTF Failed to Load: %s because %s\n",font_file, TTF_GetError());
    }
 }

////////////////////////////////////////////////////////////////////////////////
//Name:    Close Font
//Purpose: Close a font
//Notes:   Closing a font that has been declared, but not loaded causes Segfault
//         which is annoyingly hard to test for.
//Also:    You can probably get away with just calling TTF_CloseFont locally
//         as long as you haven't done something stupid, and remember to NULL it
//         if you want to avoid pointer shenanigans
//////////////////////////////////////////////////////////////////////////////// 
 void close_font(TTF_Font **font)
 {//--//
  if(font==NULL)
    {return;
    }
  //--//
  else
    {TTF_CloseFont(*font);
    }
  *font=NULL;
 }
 
////////////////////////////////////////////////////////////////////////////////
//Name:    Text Render
//Purpose: Renders text to a surface
//Notes:   DO NOT UNDER ANY CIRCUMSTANCES call this every frame. Render to an
//         Intermediary Surface, then blit the intermediary every frame
//Also:    NULL fonts and closed fonts will cause segfaults if not caught.
////////////////////////////////////////////////////////////////////////////////
 void text_render(SDL_Surface *dest_surface, int dest_x, int dest_y, TTF_Font *font, int r, int g, int b, int a, const char* text_in, ...)
 {
  //--//
  if(font==NULL)
    {printf("CRITICAL EXISTANCE FAILURE: Attempting to draw NULL font \n");
     return;
    }
  //--//
  char text[1024];
  SDL_Surface *text_surface;
  SDL_Color text_color={r,g,b,a};
  //--//
  va_list varlist;
  va_start(varlist,text_in);
  vsprintf(text,text_in,varlist);
  va_end(varlist);
  //--//
  if(!(text_surface=TTF_RenderText_Solid(font,text,text_color)))
    {printf("CRITICAL EXISTANCE FAILURE: TTF Failed to Render: %s\n",text);
    }
  //--//
  SDL_Rect dest_rect={dest_x,dest_y,0,0};
  SDL_BlitSurface(text_surface, NULL, dest_surface, &dest_rect);
  //--//
  SDL_FreeSurface(text_surface);
 }

////////////////////////////////////////////////////////////////////////////////
//Name:    Init
//Purpose: Init Text rendering
////////////////////////////////////////////////////////////////////////////////
 void init()
 {printf("DEBUG:  Attempting to Init SDL_TTF \n");
  if(TTF_Init()==-1) 
    {printf("CRITICAL EXISTANCE FAILURE: TTF did not Init: %s\n",TTF_GetError());
    }
  printf("DEBUG:  SDL_TTF Initialized sucessfully \n");
 }
////////////////////////////////////////////////////////////////////////////////
}ttf;
